# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from django.template import Context, Template
from django.utils.translation import gettext_lazy as _


from thx.core.models import UserModel, DateModel, StatusModel, OrderedModel, CleanModel

from thx.i18n.models import Language


class GenericTemplate(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug'),
    )
    language = models.ForeignKey(
        Language,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)ss',
        blank=True, null=True,
        verbose_name=_('language'),
    )
    extension = models.CharField(
        choices=settings.TEMPLATE_GENERIC_TEMPLATE_EXTENSION_CHOICES,
        blank=True,
        null=True,
        max_length=10,
        verbose_name=_('extension'),
    )
    content = models.TextField(
        blank=True,
        verbose_name=_('content'),
    )

    class Meta:
        verbose_name = _('generic template')
        verbose_name_plural = _('generic templates')
        ordering = ('ordering',)
        unique_together = (('slug', 'language', 'extension',),)
        permissions = (
            ("list_generictemplate", "Can list generic template"),
            ("detail_generictemplate", "Can detail generic template"),
            ("enable_generictemplate", "Can enable generic template"),
            ("disable_generictemplate", "Can disable generic template"),
        )

    def __str__(self):
        return self.title

    def render_content(self, context={}):
        """
        :param context: dictionary to inject inside the content.
        :return: content with variables replaced by the context.
        """
        rendered_content = Template(self.content).render(Context(context))
        return rendered_content


class EmailTemplate(GenericTemplate):
    subject = models.CharField(
        blank=True,
        max_length=150,
        verbose_name=_('subject'),
    )

    class Meta:
        verbose_name = _('email template')
        verbose_name_plural = _('email templates')
        ordering = ('ordering',)
        permissions = (
            ("list_emailtemplate", "Can list email template"),
            ("detail_emailtemplate", "Can detail email template"),
            ("enable_emailtemplate", "Can enable email template"),
            ("disable_emailtemplate", "Can disable email template"),
        )

    def render_subject(self, context={}):
        """
        :param context: dictionary to inject inside the subject.
        :return: subject with variables replaced by the context.
        """
        rendered_subject = Template(self.subject).render(Context(context))
        return rendered_subject
